import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/routines/bloc/functions_bloc/functions_bloc_bloc.dart';
import 'package:syncrow_web/pages/routines/bloc/routine_bloc/routine_bloc.dart';
import 'package:syncrow_web/pages/routines/models/ceiling_presence_sensor_functions.dart';
import 'package:syncrow_web/pages/routines/models/device_functions.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_footer.dart';
import 'package:syncrow_web/pages/routines/widgets/dialog_header.dart';
import 'package:syncrow_web/pages/routines/widgets/routine_dialogs/ceiling_sensor/ceiling_sensor_helper.dart';
import 'package:syncrow_web/pages/routines/widgets/routine_dialogs/ceiling_sensor/cps_dialog_slider_selector.dart';
import 'package:syncrow_web/pages/routines/widgets/routine_dialogs/ceiling_sensor/cps_dialog_value_selector.dart';
import 'package:syncrow_web/pages/routines/widgets/routine_dialogs/ceiling_sensor/cps_functions_list.dart';

class CeilingSensorDialog extends StatefulWidget {
  const CeilingSensorDialog({
    required this.uniqueCustomId,
    required this.functions,
    required this.deviceSelectedFunctions,
    required this.device,
    required this.dialogType,
    super.key,
  });

  final String? uniqueCustomId;
  final List<DeviceFunction> functions;
  final List<DeviceFunctionData> deviceSelectedFunctions;
  final AllDevicesModel? device;
  final String dialogType;

  @override
  State<CeilingSensorDialog> createState() => _CeilingSensorDialogState();
}

class _CeilingSensorDialogState extends State<CeilingSensorDialog> {
  late final List<CpsFunctions> _cpsFunctions;

  @override
  void initState() {
    super.initState();

    _cpsFunctions = widget.functions.whereType<CpsFunctions>().where((function) {
      if (widget.dialogType == 'THEN') {
        return function.type == 'THEN' || function.type == 'BOTH';
      }
      return function.type == 'IF' || function.type == 'BOTH';
    }).toList();
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      contentPadding: EdgeInsets.zero,
      content: BlocBuilder<FunctionBloc, FunctionBlocState>(
        builder: (context, state) {
          final selectedFunction = state.selectedFunction;

          return Container(
            width: selectedFunction != null ? 600 : 360,
            height: 450,
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.circular(20),
            ),
            padding: const EdgeInsets.only(top: 20),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                const DialogHeader('Presence Sensor Condition'),
                Expanded(child: _buildMainContent(context, state)),
                DialogFooter(
                  onCancel: () => Navigator.pop(context),
                  onConfirm: state.addedFunctions.isNotEmpty
                      ? () {
                          context.read<RoutineBloc>().add(
                                AddFunctionToRoutine(
                                  state.addedFunctions,
                                  '${widget.uniqueCustomId}',
                                ),
                              );

                          Navigator.pop(context, {
                            'deviceId': widget.functions.first.deviceId,
                          });
                        }
                      : null,
                  isConfirmEnabled: selectedFunction != null,
                ),
              ],
            ),
          );
        },
      ),
    );
  }

  Widget _buildMainContent(BuildContext context, FunctionBlocState state) {
    final selectedFunction = state.selectedFunction;
    final selectedOperationName = state.selectedOperationName;
    final selectedFunctionData = state.addedFunctions.firstWhere(
      (f) => f.functionCode == selectedFunction,
      orElse: () => DeviceFunctionData(
        entityId: '',
        functionCode: selectedFunction ?? '',
        operationName: '',
        value: null,
      ),
    );
    final selectedCpsFunctions = _cpsFunctions.firstWhere(
      (f) => f.code == selectedFunction,
      orElse: () => CpsMovementFunctions(
        deviceId: '',
        deviceName: '',
        type: '',
      ),
    );
    final operations = selectedCpsFunctions.getOperationalValues();
    final isSensitivityFunction = selectedFunction == 'sensitivity';
    final isToggleFunction = isSensitivityFunction
        ? widget.dialogType == 'THEN'
        : CeilingSensorHelper.toggleCodes.contains(selectedFunction);

    return Row(
      crossAxisAlignment: CrossAxisAlignment.stretch,
      children: [
        CpsFunctionsList(cpsFunctions: _cpsFunctions),
        if (state.selectedFunction != null)
          Expanded(
            child: isToggleFunction
                ? CpsDialogValueSelector(
                    operations: operations,
                    selectedFunction: selectedFunction ?? '',
                    selectedFunctionData: selectedFunctionData,
                    cpsFunctions: _cpsFunctions,
                    operationName: selectedOperationName ?? '',
                    device: widget.device,
                  )
                : CpsDialogSliderSelector(
                    operations: operations,
                    selectedFunction: selectedFunction ?? '',
                    selectedFunctionData: selectedFunctionData,
                    cpsFunctions: _cpsFunctions,
                    operationName: selectedOperationName ?? '',
                    device: widget.device,
                    dialogType: widget.dialogType,
                  ),
          ),
      ],
    );
  }
}
