import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/device_managment/all_devices/models/devices_model.dart';
import 'package:syncrow_web/pages/routines/bloc/functions_bloc/functions_bloc_bloc.dart';
import 'package:syncrow_web/pages/routines/models/ceiling_presence_sensor_functions.dart';
import 'package:syncrow_web/pages/routines/models/device_functions.dart';
import 'package:syncrow_web/pages/routines/widgets/slider_value_selector.dart';

class CpsDialogSliderSelector extends StatelessWidget {
  const CpsDialogSliderSelector({
    required this.operations,
    required this.selectedFunction,
    required this.selectedFunctionData,
    required this.cpsFunctions,
    required this.device,
    required this.operationName,
    required this.dialogType,
    super.key,
  });

  final List<CpsOperationalValue> operations;
  final String selectedFunction;
  final DeviceFunctionData selectedFunctionData;
  final List<CpsFunctions> cpsFunctions;
  final AllDevicesModel? device;
  final String operationName;
  final String dialogType;

  @override
  Widget build(BuildContext context) {
    return SliderValueSelector(
      currentCondition: selectedFunctionData.condition,
      dialogType: dialogType,
      sliderRange: _sliderRange,
      displayedValue: _displayText,
      initialValue: selectedFunctionData.value ?? 0,
      unit: _unit,
      onConditionChanged: (condition) => context.read<FunctionBloc>().add(
            AddFunction(
              functionData: DeviceFunctionData(
                entityId: device?.uuid ?? '',
                functionCode: selectedFunction,
                operationName: selectedFunctionData.operationName,
                condition: condition,
                value: selectedFunctionData.value,
              ),
            ),
          ),
      onSliderChanged: (value) => context.read<FunctionBloc>().add(
            AddFunction(
              functionData: DeviceFunctionData(
                entityId: device?.uuid ?? '',
                functionCode: selectedFunction,
                operationName: selectedFunctionData.operationName,
                value: value,
                condition: selectedFunctionData.condition,
              ),
            ),
          ),
      dividendOfRange: _dividendOfRange,
    );
  }

  (double, double) get _sliderRange => switch (selectedFunctionData.functionCode) {
        'moving_speed' => (0, 32),
        'sensitivity' => (0, 10),
        'space_static_val' => (0, 255),
        'space_move_val' => (0, 255),
        'moving_max_dis' => (0, 10),
        'static_max_dis' => (0, 10),
        'moving_range' => (0, 25.5),
        'presence_range' => (0, 25.5),
        'presence_judgement_threshold' => (0, 255),
        'motion_amplitude_trigger_threshold' => (0, 255),
        'perceptual_boundary' => (0, 5),
        'moving_boundary' => (0, 5),
        'moving_rigger_time' => (0, 2),
        'moving_static_time' => (0, 50),
        'none_body_time' => (0, 300),
        _ => (0, 300),
      };

  String get _displayText {
    final value = selectedFunctionData.value;
    final parsedValue = double.tryParse('$value');

    return switch (selectedFunctionData.functionCode) {
      'moving_max_dis' ||
      'static_max_dis' ||
      'moving_range' ||
      'presence_range' ||
      'perceptual_boundary' ||
      'moving_boundary' =>
        parsedValue?.toStringAsFixed(1) ?? '0',
      'moving_rigger_time' => parsedValue?.toStringAsFixed(2) ?? '0',
      'moving_static_time' ||
      'none_body_time' =>
        parsedValue?.toStringAsFixed(2) ?? '0',
      _ => '${parsedValue?.toStringAsFixed(0) ?? 0}',
    };
  }

  String get _unit {
    return switch (selectedFunctionData.functionCode) {
      'moving_max_dis' ||
      'static_max_dis' ||
      'moving_range' ||
      'presence_range' ||
      'perceptual_boundary' ||
      'moving_boundary' =>
        'M',
      'moving_rigger_time' || 'moving_static_time' || 'none_body_time' => 'sec',
      _ => '',
    };
  }

  double get _dividendOfRange => switch (selectedFunctionData.functionCode) {
        'sensitivity' => 1,
        'moving_speed' => 1,
        'space_static_val' => 1,
        'space_move_val' => 1,
        'presence_reference' => 5,
        'moving_reference' => 5,
        'moving_max_dis' => 0.5,
        'static_max_dis' => 0.5,
        'moving_range' => 0.1,
        'presence_range' => 0.1,
        'perceptual_boundary' => 0.5,
        'moving_boundary' => 0.5,
        'moving_rigger_time' => 0.1,
        'moving_static_time' => 1.0,
        'none_body_time' => 5.0,
        'sports_para' => 1.0,
        _ => 1,
      };
}
